import contextlib
from typing import Dict, Any, List

import pythoncom
import xlwings as xw
from xlwings import Sheet

from itchecker.base_checker import Checker


class ExcelChecker(Checker):
    REPORT_TEMPLATE_NAME = 'excel_report.html'
    SUPPORTED_EXTENSIONS = {'xlsx', 'xls', 'xlsm'}
    _DATA_SETUP = {
        'Значення': {'field': 'value'},
        'Курсив': {'field': 'font.italic'},
        'Жирність': {'field': 'font.bold'},
        'Колір тексту': {'field': 'font.color'},
        'Назва шрифту': {'field': 'font.name'},
        'Розмір шрифту': {'field': 'font.size'},
        'Заливка': {'field': 'color'},
        'Числовий формат': {'field': 'number_format'},
    }

    @classmethod
    def insert_data(cls, connection: Sheet, inputs: Dict[str, Any]):
        for cell, value in inputs.items():
            connection.range(cell).value = value

    @classmethod
    def acquire_data(cls, connection: Sheet, query: str, outputs: List[str]) -> Dict[str, Any]:
        ERRORS = {-2146826281: '#DIV/0!', -2146826246: '#N/A', -2146826259: '#NAME?', -2146826288: '#NULL!',
                  -2146826252: '#NUM!', -2146826265: '#REF!', -2146826273: '#VALUE!'}

        data: Dict[str, Any] = {}
        for cell in outputs:
            # connection.charts.api._inner.ShapeRange.Chart.SeriesCollection(1)
            data[cell] = Checker.getattr(connection.range(cell), cls._DATA_SETUP[query]['field'])
            if data[cell] == '':
                data[cell] = None
            if connection.range(cell).raw_value in ERRORS:
                data[cell] = {'error': ERRORS[connection.range(cell).raw_value]}
        return data

    @classmethod
    def save_test(cls, path: str, connection: Sheet, output_path: str):
        connection.book.save(output_path)

    @classmethod
    def load_inputs(cls, inputs_setup_sheet: Sheet, test_no: int) -> Dict[str, Any]:
        inputs = {}
        num_cells = inputs_setup_sheet['B1'].current_region.last_cell.row - 1
        for cell_no in range(num_cells):
            cell_address = inputs_setup_sheet[cell_no + 1, 0].value
            value = inputs_setup_sheet[cell_no + 1, test_no + 1].value
            inputs[cell_address] = value
        return inputs

    @classmethod
    @contextlib.contextmanager
    def get_connection(cls, path):
        pythoncom.CoInitialize()
        with xw.App(visible=False) as app:
            wb = app.books.open(path)
            sheet = wb.sheets.active
            try:
                yield sheet
            finally:
                wb.close()



import argparse
from concurrent.futures.thread import ThreadPoolExecutor
from os import listdir, makedirs
from os.path import join, exists

# python console_tool.py --works-folder ./works --test-file ./tests.xlsx --correct-file ./etalon.xlsx --output-folder ./output
from itchecker.access import AccessChecker
from itchecker.excel import ExcelChecker
from itchecker.word import WordChecker


__version__ = "2.0.1"


def main():
    parser = argparse.ArgumentParser()

    checkers = {
        'excel': ExcelChecker,
        'access': AccessChecker,
        'word': WordChecker,
    }

    parser.add_argument('--checker', required=True, choices=checkers.keys())
    parser.add_argument('--works-folder', required=True)
    parser.add_argument('--test-file', required=True)
    parser.add_argument('--correct-file', required=True)
    parser.add_argument('--output-folder', required=True)

    args = parser.parse_args()

    if not exists(args.output_folder):
        print(f'Створення каталогу {args.output_folder}')
        makedirs(args.output_folder)

    if not exists(join(args.output_folder, 'tests')):
        makedirs(join(args.output_folder, 'tests'))

    Checker = checkers[args.checker]

    Checker.pre_process(args.correct_file)
    tests = Checker.load_tests(args.test_file, args.correct_file, args.output_folder)
    files = [
        join(args.works_folder, f)
        for f in listdir(args.works_folder)
        if any([f.endswith(f'.{ext}') for ext in Checker.SUPPORTED_EXTENSIONS]) and not f.startswith('~')
    ]

    for file in files:
        Checker.pre_process(file)

    def test_file_job(file):
        return Checker.test_file(file, tests, args.output_folder)

    with ThreadPoolExecutor(3) as executor:
        results = executor.map(test_file_job, files)

    results = list(results)
    Checker.create_report(tests, results, args.output_folder)


if __name__ == '__main__':
    main()



if __name__ == '__main__':
    import win32com.client as win32
    import os

    word = win32.gencache.EnsureDispatch('Word.Application')
    word.Visible = True
    doc_path = os.path.join(os.path.dirname(os.path.realpath(__file__)), 'author.docx')
    doc = word.Documents.Open(doc_path)
    doc
    fields_by_tag = {control.Tag: control for control in doc.Bookmarks}
    for control in doc.ContentControls:
        if control.Tag == 'field1':
            control.Range.Text = '15'
    doc.Fields.Update()
    print(fields_by_tag['field2'].Range.Text)



import contextlib
import os
from typing import Dict, List, Any

import pythoncom
import win32com.client as win32
from xlwings import Sheet

from itchecker.base_checker import Checker


class WordInteractor:
    def __init__(self, document: 'Document'):
        self._document = document
        self._OBJECT_SETUP = {
            'поле вводу': {'name_field': 'Tag', 'object_field': 'ContentControls'},
            'таблиця': {
                'name_field': 'Title',
                'object_field': 'Tables',
                'selector': lambda obj, path: obj.Cell(int(path[2]), int(path[3]))
            },
            'закладка': {'name_field': 'Name', 'object_field': 'Bookmarks'},
            'абзац': {'name_field': 'id', 'object_field': 'Paragraphs'},
            'документ': {'object_field': 'Content'},
        }
        self._DATA_SETUP = {
            'Значення': {'field': 'Text', 'scope': 'whole'},
            'Курсив': {'field': 'Font.Italic', 'scope': 'word'},
            'Жирність': {'field': 'Font.Bold', 'scope': 'word'},
            'Колір тексту': {'field': 'Font.Color', 'scope': 'word'},
            'Назва шрифту': {'field': 'Font.Name', 'scope': 'word'},
            'Розмір шрифту': {'field': 'Font.Size', 'scope': 'word'},
            'Підрядковий': {'field': 'Font.Subscript', 'scope': 'word'},
            'Надрядковий': {'field': 'Font.Superscript', 'scope': 'word'},
            'Підкреслений': {'field': 'Font.Underline', 'scope': 'word'},
        }

    def _get_objects(self, reference: str) -> List:
        self._document.Fields.Update()
        path = reference.split(':')
        object_type = path[0]

        if object_type not in self._OBJECT_SETUP:
            raise Exception(
                f'Нерозпізнаний тип об\'єкту: {object_type}. Доступні типи: {", ".join(self._OBJECT_SETUP.keys())}.'
            )

        name_field = self._OBJECT_SETUP[object_type].get('name_field')
        object_field = self._OBJECT_SETUP[object_type].get('object_field')
        selector = self._OBJECT_SETUP[object_type].get('selector', lambda obj, _: obj)

        if not name_field:
            return [getattr(self._document, object_field)]

        objects = []
        for i, control in enumerate(getattr(self._document, object_field)):
            if name_field == 'id':
                if i + 1 == int(path[1]):
                    objects.append(selector(control, path).Range)
            elif getattr(control, name_field) == path[1]:
                objects.append(selector(control, path).Range)

        return objects

    def update(self):
        self._document.Fields.Update()

    def save_as(self, *args, **kwargs):
        self._document.SaveAs(*args, **kwargs)

    def _get_value(self, obj, data_type: str):
        if not obj:
            return None

        setup = self._DATA_SETUP[data_type]
        if setup['scope'] == 'whole':
            return Checker.getattr(obj, setup['field'])
        elif setup['scope'] == 'word':
            return [Checker.getattr(word, setup['field']) for word in obj.Words]
        else:
            raise Exception(f'Некоректний масштаб пошуку значення {setup["scope"]}. Доступні: whole, word')

    def read(self, reference: str, data_type: str):
        objects = self._get_objects(reference)
        return self._get_value(objects[0], data_type) if objects else None

    def write(self, reference: str, value: str):
        for control in self._get_objects(reference):
            control.Text = value


class WordChecker(Checker):
    REPORT_TEMPLATE_NAME = 'word_report.html'
    SUPPORTED_EXTENSIONS = {'doc', 'docx', 'docm'}

    @classmethod
    @contextlib.contextmanager
    def get_connection(cls, path):
        pythoncom.CoInitialize()
        word = win32.Dispatch('Word.Application')
        word.Visible = False
        doc = word.Documents.Open(path)
        try:
            yield WordInteractor(document=doc)
        except Exception as e:
            doc.Close(SaveChanges=False)
            raise e
        doc.Close(SaveChanges=False)

    @classmethod
    def insert_data(cls, connection: WordInteractor, inputs: Dict[str, Any]):
        for reference, value in inputs.items():
            connection.write(reference, value)

    @classmethod
    def acquire_data(cls, connection: WordInteractor, query: str, outputs: List[str]) -> Dict[str, Any]:
        connection.update()
        data: Dict[str, Any] = {}
        for reference in outputs:
            data[reference] = connection.read(reference, query)
        return data

    @classmethod
    def save_test(cls, path: str, connection: WordInteractor, output_path: str):
        connection.save_as(output_path)

    @classmethod
    def load_inputs(cls, inputs_setup_sheet: Sheet, test_no: int) -> Dict[str, Any]:
        inputs = {}
        num_cells = inputs_setup_sheet['B1'].current_region.last_cell.row - 1
        for cell_no in range(num_cells):
            cell_address = inputs_setup_sheet[cell_no + 1, 0].value
            value = inputs_setup_sheet[cell_no + 1, test_no + 1].value
            inputs[cell_address] = value
        return inputs



# -*- coding: utf-8 -*-


"""setup.py: setuptools control."""

import re

from pip._internal.req import parse_requirements
from setuptools import setup

version = re.search(
    '^__version__\s*=\s*"(.*)"',
    open('itchecker/console_tool.py').read(),
    re.M
).group(1)

with open("README.rst", "rb") as f:
    long_descr = f.read().decode("utf-8")

install_reqs = parse_requirements('itchecker/requirements.txt', session='hack')
reqs = [str(ir.requirement) for ir in install_reqs]

setup(
    name="itchecker",
    packages=["itchecker"],
    entry_points={
        "console_scripts": ['itchecker = itchecker.console_tool:main']
    },
    package_data={'': ['templates/*', 'requirements.txt']},
    version=version,
    description="Python command line application bare bones template.",
    long_description=long_descr,
    author="itolymp",
    author_email="it@itolymp.com",
    install_requires=reqs,
)



import contextlib
import shutil
import threading
import time
from os.path import abspath
from typing import Dict, List, Tuple

import pyodbc
import win32com.client
from xlwings import Sheet

from itchecker.base_checker import Checker


class AccessChecker(Checker):
    REPORT_TEMPLATE_NAME = 'access_report.html'
    SUPPORTED_EXTENSIONS = {'accdb', 'mdb'}

    @classmethod
    def pre_process(cls, path: str):
        oApp = win32com.client.Dispatch("Access.Application")
        try:
            oApp.OpenCurrentDatabase(abspath(path))
            currentdb = oApp.CurrentDb()
            for query in currentdb.QueryDefs:
                query.SQL = query.SQL.replace('"', "'")
            currentdb = None
            oApp.DoCmd.CloseDatabase()

        except Exception as e:
            print(f'Помилка препоцесингу: {e}.')

        finally:
            currentdb = None
            oApp.Quit()

    @classmethod
    @contextlib.contextmanager
    def get_connection(cls, path):
        conn_str = (
            r'DRIVER={Microsoft Access Driver (*.mdb, *.accdb)};'
            rf'DBQ={abspath(path)};'
        )
        with pyodbc.connect(conn_str) as conn:
            db_cursor = conn.cursor()
            try:
                yield db_cursor
            except Exception as e:
                db_cursor.close()
                conn.close()
                raise e
            db_cursor.close()

    @classmethod
    def insert_data(cls, connection: pyodbc.Cursor, queries: List[str]):
        for query in queries:
            connection.execute(query)

    @classmethod
    def acquire_data(cls, connection: pyodbc.Cursor, query: str, outputs: List[str]) -> Tuple[List[str], List[Dict]]:
        finished = False

        def watchdog():  # 464 204  # 460 464 562
            for i in range(360):
                time.sleep(0.1)
                if finished:
                    return
            try:
                connection.cancel()
            except:
                pass

        t = threading.Thread(target=watchdog)
        t.start()
        connection.execute(query)
        columns = [column[0] for column in connection.description]
        results = []
        for row in connection.fetchall():
            results.append(dict(zip(columns, row)))
        finished = True
        t.join()
        return outputs, [{column: result.get(column) for column in outputs} for result in results]

    @classmethod
    def save_test(cls, path: str, connection: pyodbc.Cursor, output_path: str):
        shutil.copyfile(path, output_path)

    @classmethod
    def load_inputs(cls, inputs_setup_sheet: Sheet, test_no: int) -> List[str]:
        return list(filter(
            lambda val: val is not None,
            inputs_setup_sheet.range(
                (2, test_no + 2),
                inputs_setup_sheet.range((2, test_no + 2)).end('down')
            ).value
        ))



import traceback
from abc import ABC, abstractmethod
from os.path import join, basename, dirname, abspath, splitext
from typing import Dict, Any, List, Optional

import xlwings as xw
from jinja2 import Template
from xlwings import Sheet


class Checker(ABC):
    REPORT_TEMPLATE_NAME = None
    SUPPORTED_EXTENSIONS = None

    @classmethod
    def getattr(cls, obj, attr: str):
        path = attr.split('.')
        for el in path:
            obj = getattr(obj, el)
        return obj

    @classmethod
    def pre_process(cls, path: str):
        pass

    @classmethod
    def post_process(cls):
        pass

    @classmethod
    @abstractmethod
    def insert_data(cls, connection: Any, inputs: Any):
        pass

    @classmethod
    @abstractmethod
    def acquire_data(cls, connection: Any, query: Optional[str], outputs: List[str]) -> Any:
        pass

    @classmethod
    @abstractmethod
    def save_test(cls, path: str, connection: Any, output_path: str):
        pass

    @classmethod
    @abstractmethod
    def load_inputs(cls, inputs_setup_sheet: Sheet, test_no: int) -> List[str]:
        pass

    @classmethod
    def load_query(cls, outputs_setup_sheet: Sheet, test_no: int) -> List[str]:
        return outputs_setup_sheet[1, test_no + 1].value

    @classmethod
    def get_extension(cls, path: str) -> str:
        _, file_extension = splitext(path)
        return file_extension

    @classmethod
    def load_tests(cls, test_file_path, correct_file_path, output_folder_path) -> List[Dict[str, Any]]:
        with cls.get_connection(correct_file_path) as correct_file_connection:
            with xw.App(visible=False) as app:
                setup_wb = app.books.open(test_file_path)
                inputs_setup_sheet = setup_wb.sheets['input']
                outputs_setup_sheet = setup_wb.sheets['output']
                num_tests = inputs_setup_sheet['B1'].current_region.last_cell.column - 1
                print(f'Кількість тестів: {num_tests}')

                tests: List[Dict[str, Any]] = []
                for test_no in range(num_tests):
                    current_test = {
                        'score': inputs_setup_sheet[0, test_no + 1].value,
                        'query': cls.load_query(outputs_setup_sheet, test_no),
                        'inputs': cls.load_inputs(inputs_setup_sheet, test_no),
                        'outputs': list(filter(
                            lambda val: val is not None,
                            outputs_setup_sheet.range(
                                (3, test_no + 2),
                                outputs_setup_sheet.range((3, test_no + 2)).end('down')
                            ).value
                        )),
                        'results': [],
                    }

                    cls.insert_data(correct_file_connection, current_test['inputs'])
                    cls.save_test(correct_file_path, correct_file_connection,
                                  join(output_folder_path, 'tests',
                                       f'test{test_no + 1}{cls.get_extension(correct_file_path)}'))

                    current_test['results'] = cls.acquire_data(correct_file_connection, current_test['query'],
                                                               current_test['outputs'])
                    tests.append(current_test)

                setup_wb.close()
        return tests

    @classmethod
    @abstractmethod
    def get_connection(cls, path):
        pass

    @classmethod
    def test_file(cls, path: str, tests, output_folder_path: str):
        file_name = basename(path)
        score: float = 0.0
        max_score: float = 0.0
        results: List[float] = []
        details = []

        try:
            with cls.get_connection(path) as conn:
                print(f'Початок тестування роботи: {path}')

                for test in tests:
                    test_score = 0.0
                    try:
                        cls.insert_data(conn, test['inputs'])
                        file_data = cls.acquire_data(conn, test['query'], test['outputs'])
                        test_score = int(file_data == test['results']) * test['score']
                        score += test_score
                        test_details = {
                            'score': test_score,
                            'results': file_data
                        }
                    except Exception as e:
                        # print(f'Помилка виконання тесту: {path}. Деталі: {e}')
                        test_details = {
                            'score': 0.0,
                            'results': ([], []),
                            'error': str(e)
                        }
                    max_score += test['score']
                    results.append(test_score)
                    details.append(test_details)

            print(f'Кінець тестування роботи: {path}. Кількість балів: {score}')
            return {
                'file': file_name,
                'scores': results,
                'total_score': score
            }
        except Exception as e:
            print(f'Помилка тестування роботи: {path}. Деталі: {e}')
            print(f'Кінець тестування роботи: {path}. Кількість балів: {score}')

            remaining_items = len(tests) - len(results)
            results += [0.0] * remaining_items
            details += [{
                'score': 0.0,
                'results': ([], []),
                'error': str(e)
            }] * remaining_items

            return {
                'file': file_name,
                'scores': results,
                'total_score': score,
                'error': str(e)
            }
        finally:
            with open(join(dirname(abspath(__file__)), 'templates', cls.REPORT_TEMPLATE_NAME), encoding='utf-8') as f:
                template = Template(f.read())
            with open(join(output_folder_path, f'{file_name}.html'), 'w', encoding='utf-8') as f:
                f.write(template.render(details=details, tests=tests, file_name=file_name, total_score=score,
                                        max_score=max_score))

    @classmethod
    def create_report(cls, tests, results, output_folder_path):
        with xw.App(visible=False) as app:
            wb = app.books.add()
            sheet = wb.sheets.add('results')

            sheet[0, 1].value = 'Сума'
            sheet[1, 0].value = 'Максимум'

            total_score = 0.0
            for i, test in enumerate(tests):
                sheet[0, i + 2].value = f'Тест №{i + 1}'
                sheet[1, i + 2].value = test['score']
                total_score += test['score']
            sheet[1, 1].value = total_score

            for i, result in enumerate(results):
                sheet[i + 2, 0].value = result['file']
                sheet[i + 2, 1].value = result['total_score']
                for j, score in enumerate(result['scores']):
                    sheet[i + 2, j + 2].value = score

                if 'error' in result:
                    sheet[i + 2, 1].api.AddComment(Text=f'Помилка тестування роботи: {result["error"]}')
            try:
                wb.save(join(output_folder_path, 'results.xlsx'))
            except:
                input('Помилка запису результатів. Переконайтеся, що файл результатів не відкритий в Excel та введіть Yes.')
                wb.save(join(output_folder_path, 'results.xlsx'))
            wb.close()
